﻿Imports Telerik.Windows.Controls
Imports Telerik.Windows.Controls.ScheduleView

Public Class ConflictCheckingDragDropBehavior
	Inherits ScheduleViewDragDropBehavior

	Private _importantResourceTypeName As String

	Public Property ImportantResourceTypeName() As String
		Get
			Return Me._importantResourceTypeName
		End Get
		Set(ByVal value As String)
			Me._importantResourceTypeName = value
		End Set
	End Property

	Public Overrides Function CanDrop(ByVal state As DragDropState) As Boolean
		Return state.DestinationAppointmentsSource.OfType(Of IAppointment)().Where(Function(a As IAppointment) Not state.DraggedAppointments.Contains(a)).All(Function(a As IAppointment) state.DestinationSlots.All(Function(s As Slot) Not Me.AreOverlapping(a, s)))
	End Function

	Public Overrides Function CanResize(ByVal state As DragDropState) As Boolean
		Return state.DestinationAppointmentsSource.OfType(Of IAppointment)().Where(Function(a As IAppointment) Not a.Equals(state.Appointment)).All(Function(a As IAppointment) state.DestinationSlots.All(Function(s As Slot) Not Me.AreOverlapping(a, s)))
	End Function

	Public Overrides Function CoerceDraggedItems(ByVal state As DragDropState) As IEnumerable(Of IOccurrence)
		Return state.DraggedAppointments.Where(AddressOf OwnedByCurrentUser).ToList()
	End Function

	Public Overrides Function CanStartDrag(ByVal state As DragDropState) As Boolean
		Return MyBase.CanStartDrag(state) AndAlso state.DraggedAppointments.All(AddressOf OwnedByCurrentUser)
	End Function

	Public Overrides Function CanStartResize(ByVal state As DragDropState) As Boolean
		Return MyBase.CanStartResize(state) AndAlso OwnedByCurrentUser(state.Appointment)
	End Function

	Public Overrides Sub Drop(ByVal state As DragDropState)
		MyBase.Drop(state)

		For Each a As IAppointment In state.DraggedAppointments
			Me.AddCurrentUser(state, a)
		Next
	End Sub

	Private Shared Function OwnedByCurrentUser(ByVal item As IOccurrence) As Boolean
		Dim appointment As IAppointment = If(TypeOf item Is IAppointment, TryCast(item, IAppointment), TryCast(item, Occurrence).Appointment)
		Return appointment.Resources.OfType(Of IResource)().Any(Function(r As IResource) r.ResourceType = "UserName" AndAlso r.ResourceName = User.CurrentUser)
	End Function

	Private Shared Function AreIntersected(ByVal first As IEnumerable(Of IResource), ByVal other As IEnumerable(Of IResource), ByVal importantResourceTypeName As String) As Boolean
		Dim firstResources As IEnumerable(Of IResource) = first.Where(Function(r As IResource) r.ResourceType = importantResourceTypeName).Distinct()

		Dim otherResources As IEnumerable(Of IResource) = other.Where(Function(r As IResource) r.ResourceType = importantResourceTypeName).Distinct()

		Return firstResources.Where(Function(r As IResource) otherResources.Contains(r)).Count() = firstResources.Count()
	End Function

	Private Function AreOverlapping(ByVal appointment As IAppointment, ByVal slot As Slot) As Boolean
		Return appointment.IntersectsWith(slot) AndAlso AreIntersected(appointment.Resources.OfType(Of IResource)(), slot.Resources.OfType(Of IResource)(), Me.ImportantResourceTypeName)
	End Function

	Private Sub AddCurrentUser(ByVal state As DragDropState, ByVal a As IAppointment)
		Me.BeginEdit(state, a)
		a.Resources.Add(New Resource(User.CurrentUser, "UserName"))
		Me.TryCommitEdit(state)
	End Sub


End Class
