﻿Imports Telerik.Windows.Controls
Imports Telerik.Windows.Controls.ScheduleView

Public Class EditOnlyMyAppointmentsBehavior

	Public Shared ReadOnly IsEnabledProperty As DependencyProperty = DependencyProperty.RegisterAttached("IsEnabled", GetType(Boolean), GetType(EditOnlyMyAppointmentsBehavior), New UIPropertyMetadata(AddressOf OnIsEnabledPropertyChanged))

	Public Shared Function GetIsEnabled(ByVal obj As DependencyObject) As Boolean
		Return CBool(obj.GetValue(IsEnabledProperty))
	End Function

	Public Shared Sub SetIsEnabled(ByVal obj As DependencyObject, ByVal value As Boolean)
		obj.SetValue(IsEnabledProperty, value)
	End Sub

	Private Shared Sub OnIsEnabledPropertyChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
		Dim scheduleView As RadScheduleView = TryCast(d, RadScheduleView)

		AddHandler scheduleView.AppointmentEditing, AddressOf RadScheduleView_AppointmentEditing
		AddHandler scheduleView.AppointmentDeleting, AddressOf RadScheduleView_AppointmentDeleting
		AddHandler scheduleView.ShowDialog, AddressOf RadScheduleView_ShowDialog
		AddHandler scheduleView.AppointmentCreating, AddressOf RadScheduleView_AppointmentCreating
	End Sub

	Private Shared Sub RadScheduleView_AppointmentEditing(ByVal sender As Object, ByVal e As AppointmentEditingEventArgs)
		If NotOwnedByCurrentUser(e.Appointment) Then
			e.Cancel = True
		End If
	End Sub

	Private Shared Sub RadScheduleView_AppointmentDeleting(ByVal sender As Object, ByVal e As AppointmentDeletingEventArgs)
		If NotOwnedByCurrentUser(e.Appointment) Then
			e.Cancel = True
		End If
	End Sub

	Private Shared Sub RadScheduleView_ShowDialog(ByVal sender As Object, ByVal e As ShowDialogEventArgs)
		Dim appointments As IEnumerable(Of IOccurrence) = Nothing

		Dim confirmDialog As ConfirmDialogViewModel = TryCast(e.DialogViewModel, ConfirmDialogViewModel)
		If confirmDialog IsNot Nothing Then
			appointments = confirmDialog.Appointments
		End If

		Dim editDialog As AppointmentDialogViewModel = TryCast(e.DialogViewModel, AppointmentDialogViewModel)
		If editDialog IsNot Nothing Then
			appointments = New IOccurrence() {editDialog.Occurrence.Appointment}
		End If

		Dim recurrenceChoiceDialog As RecurrenceChoiceDialogViewModel = TryCast(e.DialogViewModel, RecurrenceChoiceDialogViewModel)
		If recurrenceChoiceDialog IsNot Nothing Then
			appointments = New IAppointment() {recurrenceChoiceDialog.Occurrence.Appointment}
		End If

		If appointments IsNot Nothing AndAlso appointments.Any(AddressOf NotOwnedByCurrentUser) Then
			e.Cancel = True
			e.DefaultDialogResult = False
		End If
	End Sub

	Private Shared Sub RadScheduleView_AppointmentCreating(ByVal sender As Object, ByVal e As AppointmentCreatingEventArgs)
		e.Appointment.Resources.Add(New Resource(User.CurrentUser, "UserName"))
	End Sub

	Private Shared Function NotOwnedByCurrentUser(ByVal item As IOccurrence) As Boolean
		Dim appointment As IAppointment = If(TypeOf item Is IAppointment, TryCast(item, IAppointment), TryCast(item, Occurrence).Appointment)
		Return appointment.Resources.OfType(Of IResource)().Any(Function(r As IResource) r.ResourceType = "UserName" AndAlso r.ResourceName <> User.CurrentUser)
	End Function

End Class
