﻿using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ScheduleView_CustomViewDefinition
{
	public static class EditOnlyMyAppointmentsBehavior
	{
#if WPF
		public static readonly DependencyProperty IsEnabledProperty =
			DependencyProperty.RegisterAttached("IsEnabled", typeof(bool), typeof(EditOnlyMyAppointmentsBehavior), new UIPropertyMetadata(OnIsEnabledPropertyChanged));
#else
        public static readonly DependencyProperty IsEnabledProperty =
			DependencyProperty.RegisterAttached("IsEnabled", typeof(bool), typeof(EditOnlyMyAppointmentsBehavior), new PropertyMetadata(OnIsEnabledPropertyChanged));
#endif
		public static bool GetIsEnabled(DependencyObject obj)
		{
			return (bool)obj.GetValue(IsEnabledProperty);
		}

		public static void SetIsEnabled(DependencyObject obj, bool value)
		{
			obj.SetValue(IsEnabledProperty, value);
		}

		private static void OnIsEnabledPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			RadScheduleView scheduleView = d as RadScheduleView;

			scheduleView.AppointmentEditing += RadScheduleView_AppointmentEditing;
			scheduleView.AppointmentDeleting += RadScheduleView_AppointmentDeleting;
			scheduleView.ShowDialog += RadScheduleView_ShowDialog;
			scheduleView.AppointmentCreating += RadScheduleView_AppointmentCreating;
		}

		private static void RadScheduleView_AppointmentEditing(object sender, AppointmentEditingEventArgs e)
		{
			if (NotOwnedByCurrentUser(e.Appointment))
			{
				e.Cancel = true;
			}
		}

		private static void RadScheduleView_AppointmentDeleting(object sender, AppointmentDeletingEventArgs e)
		{
			if (NotOwnedByCurrentUser(e.Appointment))
			{
				e.Cancel = true;
			}
		}

		private static void RadScheduleView_ShowDialog(object sender, ShowDialogEventArgs e)
		{
			IEnumerable<IOccurrence> appointments = null;

			ConfirmDialogViewModel confirmDialog = e.DialogViewModel as ConfirmDialogViewModel;
			if (confirmDialog != null)
			{
				appointments = confirmDialog.Appointments;
			}

			AppointmentDialogViewModel editDialog = e.DialogViewModel as AppointmentDialogViewModel;
			if (editDialog != null)
			{
				appointments = new IOccurrence[] { editDialog.Occurrence.Appointment };
			}

			RecurrenceChoiceDialogViewModel recurrenceChoiceDialog = e.DialogViewModel as RecurrenceChoiceDialogViewModel;
			if (recurrenceChoiceDialog != null)
			{
				appointments = new IAppointment[] { recurrenceChoiceDialog.Occurrence.Appointment };
			}

			if (appointments != null && appointments.Any(NotOwnedByCurrentUser))
			{
				e.Cancel = true;
				e.DefaultDialogResult = false;
			}
		}

		private static void RadScheduleView_AppointmentCreating(object sender, AppointmentCreatingEventArgs e)
		{
			e.Appointment.Resources.Add(new Resource(User.CurrentUser, "UserName"));
		}

		private static bool NotOwnedByCurrentUser(IOccurrence item)
		{
			IAppointment appointment = item is IAppointment ? item as IAppointment : (item as Occurrence).Appointment;
			return appointment.Resources.OfType<IResource>().Any((IResource r) => r.ResourceType == "UserName" && r.ResourceName != User.CurrentUser);
		}
	}
}
