﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;


namespace ScheduleView_CustomViewDefinition
{
	public class ExampleViewModel : ViewModelBase
	{
		private Uri appointmentsSource;
		private ObservableCollection<Appointment> appointments;

		public Uri AppointmentsSource
		{
			get { return this.appointmentsSource; }
			set { this.appointmentsSource = value; }
		}

		public ObservableCollection<Appointment> Appointments
		{
			get
			{
				if (this.appointments == null)
				{
					this.appointments = new ObservableCollection<Appointment>(LoadAppointmentsSource(this.AppointmentsSource));
				}
				return this.appointments;
			}
		}

		protected static IEnumerable<Appointment> LoadAppointmentsSource(Uri appointmentsSource)
		{
			if (appointmentsSource != null)
			{

				IEnumerable<Appointment> appointments = Application.LoadComponent(appointmentsSource) as IEnumerable<Appointment>;

				DateTime firstDate = appointments.Min(new Func<Appointment, DateTime>(GetStart));

				DayOfWeek firstDay = CultureInfo.CurrentCulture.DateTimeFormat.FirstDayOfWeek;

				DateTime firstDayOfCurrentWeek = DateTime.Today.Subtract(TimeSpan.FromDays((int)DateTime.Today.DayOfWeek - (int)firstDay));

				TimeSpan offset = firstDayOfCurrentWeek - firstDate;

				foreach (IAppointment appointment in appointments)
				{
					appointment.Start += offset;
					appointment.End += offset;
				}

				return appointments;
			}
			return Enumerable.Empty<Appointment>();
		}

		private static DateTime GetStart(Appointment a)
		{
			return a.Start.Date;
		}
	}
}