﻿using System;
using System.Globalization;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ScheduleView_CustomViewDefinition
{
	public class MyGamesViewDefinition : CustomViewDefinition
	{
#if WPF
		static MyGamesViewDefinition()
		{
			StretchGroupHeadersProperty.OverrideMetadata(typeof(MyGamesViewDefinition), new FrameworkPropertyMetadata(true));
			VisibleDaysProperty.OverrideMetadata(typeof(MyGamesViewDefinition), new FrameworkPropertyMetadata(7));
			LargeChangeIntervalProperty.OverrideMetadata(typeof(MyGamesViewDefinition), new FrameworkPropertyMetadata(DateTimeInterval.FromDays(7)));
		}
#else
        public MyGamesViewDefinition()
        {
            this.StretchGroupHeaders = true;
            this.VisibleDays = 7;
            this.LargeChangeInterval = DateTimeInterval.FromDays(7);
        }
#endif

		protected override string FormatVisibleRangeText(System.IFormatProvider formatInfo, System.DateTime rangeStart, System.DateTime rangeEnd, System.DateTime currentDate)
		{
			string format = rangeStart.Year == rangeEnd.Year ?
							(rangeStart.Month == rangeEnd.Month ?
							"{0:d } - {1:d MMMM yyyy}" :
							"{0:d MMMM} - {1:d MMMM yyyy}") :
							"{0:d MMMM yyyy} - {1:d MMMM yyyy}";

			return string.Format(formatInfo, format, rangeStart, rangeEnd);
		}

		protected override DateTime GetVisibleRangeStart(DateTime currentDate, CultureInfo culture, DayOfWeek? firstDayOfWeek)
		{
			if (culture == null)
			{
				culture = CultureInfo.InvariantCulture;
			}

			DayOfWeek dayOfWeek = firstDayOfWeek ?? culture.DateTimeFormat.FirstDayOfWeek;

			return CalendarHelper.GetFirstDayOfWeek(currentDate, dayOfWeek);
		}

		protected override DateTime GetVisibleRangeEnd(DateTime currentDate, CultureInfo culture, DayOfWeek? firstDayOfWeek)
		{
			if (culture == null)
			{
				culture = CultureInfo.InvariantCulture;
			}

			DayOfWeek dayOfWeek = firstDayOfWeek ?? culture.DateTimeFormat.FirstDayOfWeek;
			if (this.DayEndTime > this.DayStartTime)
			{
				return CalendarHelper.GetFirstDayOfWeek(currentDate, dayOfWeek).AddDays(this.VisibleDays - 1).Add(this.DayEndTime);
			}

			return CalendarHelper.GetFirstDayOfWeek(currentDate, dayOfWeek).AddDays(this.VisibleDays).Add(this.DayEndTime);
		}
	}
}